@@ -1,4 +1,4 @@
-#!/usr/bin/env python2.3
+#!/usr/bin/env python2
 '''Utility classes and functions for getmail.
 '''
 
@@ -27,6 +27,7 @@ __all__ = [
     'uid_of_user',
     'updatefile',
     'get_password',
+    'run_command',
 ]
 
 
@@ -44,6 +45,12 @@ import grp
 import getpass
 import commands
 import sys
+import tempfile
+import errno
+try:
+    import subprocess
+except ImportError, o:
+    subprocess = None
 
 # hashlib only present in python2.5, ssl in python2.6; used together
 # in SSL functionality below
@@ -551,21 +558,27 @@ def check_ssl_version(conf):
             'specifying ssl_version not supported by this installation of '
             'Python; requires Python 2.6'
         )
+    def get_or_fail(version, symbol):
+        if symbol is not None:
+            v = getattr(ssl, symbol, None)
+            if v is not None:
+                return v
+        raise getmailConfigurationError(
+            'unknown or unsupported ssl_version "%s"' % version
+        )
+
     ssl_version = ssl_version.lower()
     if ssl_version == 'sslv23':
-        return ssl.PROTOCOL_SSLv23
+        return get_or_fail(ssl_version, 'PROTOCOL_SSLv23')
     elif ssl_version == 'sslv3':
-        return ssl.PROTOCOL_SSLv3
+        return get_or_fail(ssl_version, 'PROTOCOL_SSLv3')
     elif ssl_version == 'tlsv1':
-        return ssl.PROTOCOL_TLSv1
+        return get_or_fail(ssl_version, 'PROTOCOL_TLSv1')
     elif ssl_version == 'tlsv1_1' and 'PROTOCOL_TLSv1_1' in dir(ssl):
-        return ssl.PROTOCOL_TLSv1_1
+        return get_or_fail(ssl_version, 'PROTOCOL_TLSv1_1')
     elif ssl_version == 'tlsv1_2' and 'PROTOCOL_TLSv1_2' in dir(ssl):
-        return ssl.PROTOCOL_TLSv1_2
-    else:
-        raise getmailConfigurationError(
-            'unknown or unsupported ssl_version'
-        )
+        return get_or_fail(ssl_version, 'PROTOCOL_TLSv1_2')
+    return get_or_fail(ssl_version, None)
 
 #######################################
 def check_ssl_fingerprints(conf):
@@ -696,3 +709,42 @@ def get_password(label, user, server, protocol, logger):
         # no password found (or not on OSX), prompt in the usual way
         password = getpass.getpass('Enter password for %s:  ' % label)
     return password
+
+
+#######################################
+def run_command(command, args):
+    # Simple subprocess wrapper for running a command and fetching its exit 
+    # status and output/stderr.
+    if args is None:
+        args = []
+    if type(args) == tuple:
+        args = list(args)
+
+    # Programmer sanity checks
+    assert type(command) in (str, unicode), (
+        'command is %s (%s)' % (command, type(command))
+    )
+    assert type(args) == list, (
+        'args is %s (%s)' % (args, type(args))
+    )
+    for arg in args:
+        assert type(arg) in (str, unicode), 'arg is %s (%s)' % (arg, type(arg))
+
+    stdout = tempfile.TemporaryFile()
+    stderr = tempfile.TemporaryFile()
+
+    cmd = [command] + args
+
+    try:
+        p = subprocess.Popen(cmd, stdout=stdout, stderr=stderr)
+    except OSError, o:
+        if o.errno == errno.ENOENT:
+            # no such file, command not found
+            raise getmailConfigurationError('Program "%s" not found' % command)
+        #else:
+        raise
+
+    rc = p.wait()
+    stdout.seek(0)
+    stderr.seek(0)
+    return (rc, stdout.read().strip(), stderr.read().strip())
